using System;
using System.Collections.Generic;
using Telerik.OpenAccess.Metadata;
using Telerik.OpenAccess.Metadata.Fluent;
using Telerik.OpenAccess.Metadata.Fluent.Advanced;

namespace FluentVerticalInheritance
{
    public class FluentVerticalInheritanceMetadataSource : FluentMetadataSource
    {
        protected override IList<MappingConfiguration> PrepareMapping()
        {
            // Getting Started with the Fluent Mapping API
            // http://documentation.telerik.com/openaccess-orm/developers-guide/code-only-mapping/fluent-mapping-overview

            List<MappingConfiguration> configurations = new List<MappingConfiguration>();
            MappingConfiguration<Animal> animal = new MappingConfiguration<Animal>();
            animal.MapType().ToTable( "Animal" );
            animal.HasProperty( x => x.AnimalId ).IsIdentity( KeyGenerator.Autoinc );
            animal.HasDiscriminator().ToColumn("discrim");

            MappingConfiguration<Dog> dog = new MappingConfiguration<Dog>();
            dog.MapType( x => new
                                    {
                                        BB = x.BestFriend
                                    } )                
                .Inheritance( Telerik.OpenAccess.InheritanceStrategy.Vertical )                
                .ToTable( "Dog" );
            dog.HasDiscriminatorValue("123");

            MappingConfiguration<Cat> cat = new MappingConfiguration<Cat>();
            cat.MapType( x => new
                                    {
                                        LL = x.LivesLeft
                                    } )
                .Inheritance( Telerik.OpenAccess.InheritanceStrategy.Vertical )
                .ToTable( "Cat" );
            cat.HasDiscriminatorValue("124");

            configurations.Add( animal );
            configurations.Add( dog );
            configurations.Add( cat );
            return configurations;

        }
    }
}
